	
#version 330

in vec2 fragmentUV;
in vec3 P;
in vec3 N;
in mat3 localSurface2World;

out vec4 fragColor;

uniform sampler2D colorTexture;
uniform sampler2D normalMap;

//uniform samplerCube cm_z_tex;
//uniform vec2 near_far;

const int lightCount = 1;
uniform vec3 lightPositions[lightCount];
uniform vec3 cameraPosition;
//uniform vec3 objectColor;



uniform vec3 _SpecColor = vec3(1,1,1);
uniform float _Shininess = 200;
uniform vec3 _LightColor0 = vec3(1.0,1.0,1.0);




void main() {
	float smallest_dist_to_light;
	float curr_fragment_dist_to_light;

	vec3 textureColor = texture(colorTexture, fragmentUV).rgb;// load Normap Map
	
	vec3 normalMap_values = texture(normalMap, fragmentUV).rgb;

	// calculate local coordinates (x,y,z) of normal (from the normal map)
	vec3 normalMap_vector = vec3((2 * normalMap_values.x) - 1, (2 * normalMap_values.y) - 1, (2 * normalMap_values.z) - 1);

	// transform local normal with TBN Matrix
    vec3 normalDirection =  normalize(localSurface2World * normalMap_vector);

	vec3 viewDirection = normalize(cameraPosition - vec3(P));
    vec3 lightDirection;
    float attenuation;

	vec3 ambientLighting =  vec3(0.9f,0.9f,0.9f) * vec3(1.0f,1.0f,1.0f);

	  // initialize total lighting with ambient lighting
    vec3 totalLighting = ambientLighting;

	for (int index = 0; index < lightCount; index++) { // for all light sources

		vec3 vertexToLightSource = vec3(lightPositions[index] - P);
		float distance = length(vertexToLightSource);
		attenuation = 1.0 / distance; // linear attenuation 
		lightDirection = normalize(vertexToLightSource);
	
		vec3 diffuseReflection = attenuation * vec3(_LightColor0) * vec3(1.0f,1.0f,1.0f) 
				   * max(0.0, dot(normalDirection, lightDirection));

		 vec3 specularReflection;
				if (dot(normalDirection, lightDirection) < 0.0) 
				   // light source on the wrong side?
				{
				   specularReflection = vec3(0.0, 0.0, 0.0); 
					  // no specular reflection
				}
				else // light source on the right side
				{
				   specularReflection = attenuation * vec3(_LightColor0) 
					  * vec3(_SpecColor) * pow(max(0.0, dot(
					  reflect(-lightDirection, normalDirection), 
					  viewDirection)), _Shininess);
				} 

		totalLighting = totalLighting + diffuseReflection + specularReflection;
		// totalIntensity = ambientLighting + diffuseReflection + specularReflection;

		
		//vec3 cm_lookup_vec = P.xyz - lightPositions[index].xyz;

		//calculate shadow
		//smallest_dist_to_light = texture(cm_z_tex, cm_lookup_vec).r;
		//smallest_dist_to_light *= near_far.y;

		//curr_fragment_dist_to_light = length(cm_lookup_vec);
	}
	//float eps = 0.15;
	//fragColor = vec4(totalIntensity + ambientLighting, 1.0) * vec4(textureColor, 1.0);
	//if(smallest_dist_to_light + eps < curr_fragment_dist_to_light)
	//	fragColor = vec4(totalLighting, 1.0) * vec4(textureColor, 1.0) * 0.8f;
	//else
	//	fragColor = vec4(totalLighting, 1.0) * vec4(textureColor, 1.0);
	
	
	//fragColor.a = transparency;
	fragColor = vec4(totalLighting,1.0f) * vec4(textureColor, 1.0f);
	//fragColor = vec4(totalLighting,1.0f);

}
	
	
